#!/usr/bin/python
import gtk
import ConfigParser
import sys
import csv
import os
import time

# Case-sensitive item config parser
class CaseConfigParser(ConfigParser.RawConfigParser):
    def optionxform(self, optionstr):
        return optionstr #Yes, that's really all that's required...

class DataButton(gtk.Button): # I want to have a button in which I can store the LED strings
    def __init__(self, label, data=None, *args, **kwargs):
        gtk.Button.__init__(self, label, *args, **kwargs)
        self.data = data

    def set_data(self,data):
        self.data = data

    def get_data(self):
        return self.data

class mcedit:
    def __init__(self):
        #First, read mce.ini
        self.cp = CaseConfigParser()
        if sys.platform == "win32": #I'm testing it on my windows computer
            self.cp.read("mce.ini")
        else:
            self.cp.read("/etc/mce/mce.ini") # This is the normal mce.ini location
        self.sects = self.cp.sections()
        self.sects.sort()
        self.data = {}
        prettysections = {'DevLock': 'Device Lock', 'HomeKey': "Home Key", 'KeyPad': "Key Pad", 'LED': "Generic LED",
                          'LEDPatternMonoRX34': "Mono LED (n800)", 'LEDPatternNJoyRX44': "NJoy LED (n810)",
                          'LEDPatternNJoyRX48': "NJoy LED (n810w)", 'Modules': "Modules", 'PowerKey': "Power Key",
                          'SoftPowerOff': "Soft Power Off", 'TKLock': "Touchscreen/Key Lock"}
        #Hard-coding these isn't the greatest idea... Ideally, they'd be stored in data.csv, with everything else
        # I just can't figure out the logistics of how to do that...
        for s in self.sects:
            self.data[s.lower()] = {'pretty':prettysections[s]}

        #Now, load up the help
        self.load_help()

        # Set up a GTK Window, HPaned, and VBox        
        self.w = gtk.Window()
        self.h = gtk.HPaned()
        self.table = None
        self.v = gtk.VBox()

        # Set up the left side of the HPaned -- TreeView to list sections        
        self.t = gtk.TreeView()
        self.c = gtk.TreeViewColumn("section", gtk.CellRendererText(), text=0)
        self.t.append_column(self.c)
        self.model = gtk.ListStore(str)
        self.t.set_model(self.model)
        for s in self.sects:
            self.model.append([self.data[s.lower()]['pretty'],])
        self.t.connect("cursor-changed", self.load_prefs)
        # Set up the left side of the HPaned -- button bar
        self.b = gtk.Toolbar()
        self.b.set_style(gtk.TOOLBAR_ICONS)
        #self.b.set_icon_size(gtk.ICON_SIZE_BUTTON)
        self._toolbar()
        # Set up the left side of the HPaned -- VBox
        self.v.add(self.t)
        self.v.add(self.b)
        self.v.set_child_packing(self.b, 0, 0, 0, gtk.PACK_END)

        # Set up the right side of the HPaned -- Scrolled window to hold table            
        self.s = gtk.ScrolledWindow()
        self.s.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC) # Hide the scrollbars when there's nothing to scoll
        self.w.add(self.h)
        self.h.add(self.v)
        self.h.add(self.s)
        self.w.connect('destroy', gtk.main_quit)
        self.w.resize(800,480)
        self.w.show_all()       

    def _toolbar(self):
        # Save Button
        button = gtk.ToolButton(gtk.STOCK_SAVE)
        button.connect("clicked", self.save)
        self.b.insert(button, 0)

        # Separator
        sep = gtk.SeparatorToolItem()
        sep.set_draw(False)
        sep.set_expand(True)
        self.b.insert(sep,-1)

        # Revert Button
        button = gtk.ToolButton(gtk.STOCK_REVERT_TO_SAVED)
        button.connect("clicked", self.revert)
        self.b.insert(button,-1)

        # Another separator
        sep = gtk.SeparatorToolItem()
        sep.set_draw(False)
        sep.set_expand(True)
        self.b.insert(sep,-1)
        
        # Help Button
        self.help_button = gtk.ToggleToolButton(gtk.STOCK_HELP)
        self.help_button.connect("clicked", self.help_clicked)
        self.b.insert(self.help_button, -1)
        
        self.help = False

    def save(self,widget):
        # There's no going back after this... are we SURE?
        dlg = gtk.MessageDialog(None,0,gtk.MESSAGE_QUESTION,gtk.BUTTONS_YES_NO,"Are you sure you want to overwrite mce.ini? This cannot be undone!")
        def close(widget,response):
            if response == gtk.RESPONSE_YES:
                if sys.platform == 'win32':
                    self.cp.write(open('newmce.ini','w')) # On my windows box, I always want the original
                else:
                    self.cp.write(open('/tmp/mce.ini','w'))
                    os.system('sudo python /usr/share/mcedit/mcemv.py --save')
            widget.destroy()
        dlg.connect("response", close)
        dlg.run()

    def revert(self,widget):
        #Will be eerily similar to save...
        dlg = gtk.MessageDialog(None,0,gtk.MESSAGE_QUESTION,gtk.BUTTONS_YES_NO,"Are you sure you want to revert to the default mce.ini? You will lose all your changes!")
        def close(widget,response):
            if response == gtk.RESPONSE_YES:
                if sys.platform == 'win32':
                    pass #Actually, I never overwrite the original on windows
                else:
                    os.system('sudo python /usr/share/mcedit/mcemv.py --revert') # Gasp! Two things in one file?!?
                    # Actually, the goal would be to get rid of mcemv.py altogether, and just have everything in here
                    # TODO: Just have everything in here
            widget.destroy()
        dlg.connect("response", close)
        dlg.run()

    def help_clicked(self,widget):
        self.help = widget.get_active()

    def load_prefs(self, widget, a=None, b=None):
        #Remove the previous table from the scrolled window
        if self.s.get_child() != None:
            self.s.remove(self.s.get_child())
        if a == None or b == None:
            (a,b) = self.t.get_cursor()
        items = self.cp.items(self.sects[a[0]])
        self.table = gtk.Table(len(items),3, False)
        self.table.set_row_spacings(5)
        row = 0
        for (item,value) in items:
            eb = gtk.EventBox()
            label = gtk.Label(self.data[self.sects[a[0]].lower()][item.lower()]['pretty'])
            eb.add(label)
            eb.set_events(gtk.gdk.BUTTON_PRESS_MASK)
            eb.connect("button_press_event", self.show_help, self.data[self.sects[a[0]].lower()][item.lower()]['help'])
            type = self.data[self.sects[a[0]].lower()][item.lower()]['type']
            if type == 'str':
                edit = gtk.Entry()
                edit.set_text(value)
                edit.connect('changed',self.change_option,self.sects[a[0]],item)
            elif type == 'int':
                edit = gtk.SpinButton(gtk.Adjustment(float(value),0,999999,1,10))
                edit.connect('value-changed',self.change_option,self.sects[a[0]],item)
            elif type[0:5] == "multi":
                s = type.split("|")
                p = s[1].split(";") # Get the allowed values
                edit = gtk.combo_box_new_text()
                rem=0
                j=0
                for i in p:
                    #if i == self.data[self.sects[a[0]].lower()][item.lower()]['default']:
                    #    rem=j
                    if i == value:
                        rem = j
                    edit.append_text(i)
                    j = j+1
                edit.set_active(rem)
                edit.connect('changed', self.change_option, self.sects[a[0]],item)
            elif type == "bool":
                edit = gtk.CheckButton()
                edit.set_active(int(value))
                edit.connect('toggled',self.change_option,self.sects[a[0]],item)
            elif type == "led":
                edit = DataButton("Edit Mono LED Pattern", value)
                edit.connect("clicked", self.edit_mono_led, self.sects[a[0]], item)
            elif type == "ledmulti":
                edit = DataButton("Edit NJoy LED Pattern", value)
                edit.connect("clicked", self.edit_multi_led, self.sects[a[0]], item)
            else:
                edit = gtk.Entry()
                edit.set_text(value)
                edit.connect('changed',self.change_option,self.sects[a[0]],item)
            b = gtk.Button("D")
            b.connect('clicked', self.set_default, self.data[self.sects[a[0]].lower()][item.lower()]['default'], edit)
            self.table.attach(eb,0,1,row,row+1, yoptions=gtk.SHRINK)
            self.table.attach(edit,1,2,row,row+1, yoptions=gtk.SHRINK)
            self.table.attach(b, 2,3,row,row+1, yoptions=gtk.SHRINK)
            row = row+1
        self.table.show_all()
        self.s.add_with_viewport(self.table)

    def edit_mono_led(self,widget,section,item):
        #We need some temp variables here...
        (priority, screen, timeout, on, off, intensity) = widget.get_data().split(";") # The joys of python :D
        dlg = gtk.Dialog("Edit Mono LED Pattern", None, gtk.DIALOG_MODAL, (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dlgvbox = dlg.get_child()
        tbl = gtk.Table(5,2,False)

        # Top-left, priority label        
        lbl = gtk.Label("Priority\n(255 is lowest)")
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        tbl.attach(lbl, 0, 1, 0, 1)

        # Top-right, priority slider
        pri = gtk.HScale(gtk.Adjustment(float(priority),0,255,1,10))
        pri.set_inverted(True)
        pri.set_digits(0)
        tbl.attach(pri, 1, 2, 0, 1)

        #Mid-left, mid-right, screen option dropdown
        so = gtk.combo_box_new_text()
        so.append_text("Only when display is off")
        so.append_text("Even when display is on")
        so.append_text("Only if display is off, including acting dead")
        so.append_text("Even if display is on, including acting dead")
        so.append_text("Only when display is off, or acting dead")
        so.set_active(int(screen))
        tbl.attach(so, 0, 2, 1, 2)

        # Mid-left, timeout label
        lbl = gtk.Label("Display pattern for x seconds:\n(0 is infinite)")
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        tbl.attach(lbl, 0, 1, 2, 3)

        # Mid-right, timeout spin button
        # I used a slider for priority because it provides a good visual representation in realtion to other items
        # That isn't really needed here, so we can get away with a spin button
        time = gtk.SpinButton(gtk.Adjustment(float(timeout), 0, 86400, 1, 10))
        # Unfortunately, I have to put an upper limit on this.  So, I used 86400 = 1 day
        tbl.attach(time, 1, 2, 2, 3)

        # Mid-left, on period label
        lbl = gtk.Label("Turn LED on for x seconds:")
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        tbl.attach(lbl, 0, 1, 3, 4)

        # Mid-right, on period... spin button?
        # I think a spin button will be good.  I'll play around with it. Pretty easy to swap for a slider
        onp = gtk.SpinButton(gtk.Adjustment(float(on)/1000.0, 0.0, 10.0, 0.001, 1.0), 1, 3)
                                            # We all float on...
        tbl.attach(onp, 1, 2, 3, 4) # 1 2 3 4!

        # Mid-left, off period label
        lbl = gtk.Label("Turn LED off for x seconds:\n(0 means never turn off)")
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        tbl.attach(lbl, 0, 1, 4, 5)

        # Mid-right, off period spin button (see on period spin button notes)
        offp = gtk.SpinButton(gtk.Adjustment(float(off)/1000.0, 0.0, 10.0, 0.001, 1.0), 1, 3)
        tbl.attach(offp, 1, 2, 4, 5)

        #Bottom-left, intensity label
        lbl = gtk.Label("LED Intensity:")
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        tbl.attach(lbl, 0, 1, 5, 6)

        # Bottom-right, intensity slider
        # Again, here I think a slider is a better visual representation of the number
        inten = gtk.HScale(gtk.Adjustment(float(intensity), 0, 15, 1, 5))
        inten.set_digits(0)
        tbl.attach(inten, 1, 2, 5, 6)

        dlgvbox.pack_start(tbl)
        tbl.show_all()

        def respond(dialog, response, pri, so, time, onp, offp, inten, button, section, item):
            if response == gtk.RESPONSE_ACCEPT:
                newdata = ";".join((str(int(pri.get_value())), str(int(so.get_active())), str(int(time.get_value())), str(int(onp.get_value()*1000.0)), str(int(offp.get_value()*1000.0)), str(int(inten.get_value()))))
                # ^ Yes, I know it's messy...
                button.set_data(newdata)
                self.cp.set(section, item, newdata)
            dialog.destroy()
        dlg.connect("response", respond, pri, so, time, onp, offp, inten, widget, section, item)        
        dlg.run()

    def edit_multi_led(self,widget,section,item):
        self.PopUp(widget.get_data())
        
    def change_option(self,edit,section,item):
        if type(edit) == type(gtk.SpinButton()) or type(edit) == type(gtk.Entry()):
            self.cp.set(section,item,edit.get_text())
        elif type(edit) == type(gtk.ComboBox()):
            self.cp.set(section,item,edit.get_model()[edit.get_active()][0])
        elif type(edit) == type(gtk.CheckButton()):
            if edit.get_active() == True:
                self.cp.set(section,item,"1")
            else:
                self.cp.set(section,item,"0")

    def set_default(self,widget,default,edit):
        if type(edit) == type(gtk.SpinButton()) or type(edit) == type(gtk.Entry()):
            edit.set_text(default)
        elif type(edit) == type(gtk.ComboBox()):
            rem = 0
            j = 0
            for i in edit.get_model():
                if i[0] == default:
                    rem = j
                j = j+1
            edit.set_active(rem)
        elif type(edit) == type(gtk.CheckButton()):
            edit.set_active(int(default))
        elif type(edit) == type(DataButton("")):
            edit.set_data(default)

    def show_help(self,widget,d,help):
        if self.help == True:
            self.PopUp(help, gtk.MESSAGE_INFO)
            self.help = False
            self.help_button.set_active(False)

    def PopUp(self,msg,type=gtk.MESSAGE_ERROR):
        error_dialog = gtk.MessageDialog(None,0,type,gtk.BUTTONS_CLOSE,msg)
        error_dialog.connect("response", lambda w,e: error_dialog.destroy())
        error_dialog.run()

    def load_help(self):
        if sys.platform == "win32":
            data = csv.DictReader(open("data.csv"))
        else:
            data = csv.DictReader(open("/usr/share/mcedit/data.csv"))
        mydata = []
        #Perhaps we would like to use it in more than just this loop?
        for d in data:
            mydata.append(d)        
        for d in mydata:
            self.data[d['section'].lower()][d['name'].lower()] = d


mc = mcedit()
gtk.main()
